unit RTVDtool;

{ --- Compiler options --- }
{$B-} { Short-circuit Boolean expression evaluation }
{$V-} { Relaxed var-string checking }
{$X+} { Extended syntax }

interface
Uses
  DefLib,
  Classes, SysUtils, Graphics;

const
  C_ProgIdn     = 'RTVD';
  C_ProgTitle   = 'Radio and TV reminder';
  C_ProgVersion = 'v1.7';
  C_Copyright   = 'Copyright  1996-2000, J.R. Ferguson';
  C_AuthorEmail = 'j.r.ferguson@iname.com';
  C_AuthorURL   = 'http://hello.to/ferguson';
  C_IniFileExt  = '.INI';
  C_ChnFileExt  = '.RTC';
  C_ClipTag     = 'RTVD16';

  C_DtePos      = 01;
  C_DteLen      = 10;
  C_Tm1Pos      = 12;
  C_Tm2Pos      = 18;
  C_TimLen      = 05;
  C_DurLen      = 03;
  C_ChnPos      = 24;
  C_ChnLen      = 04;
  C_RecPos      = 29;
  C_RecLen      = 01;
  C_TtlPos      = 31;
  C_TtlLen      = MaxStp;
  C_PrnMargin   = '   ';

  HC_General    = 20;
  HC_MainData   = 30;
  HC_ChnData    = 40;

type
  T_DteStp      = String[C_DteLen];
  T_TimStp      = String[C_TimLen];
  T_DurStp      = String[C_DurLen];
  T_ChnStp      = String[C_ChnLen];
  T_TtlStp      = String[C_TtlLen];
  T_RecStp      = String[C_RecLen];

  T_State       = (St_None,St_Edit,St_Find,St_Read,St_Save,
                   St_Print,St_PrSet);

  T_Options     = record
    FileName    : TFileName;
    OptDuration : boolean;
    OptToolbar  : boolean;
    OptHints    : boolean;
    WndLeft     : integer;
    WndTop      : integer;
    WndHeight   : integer;
    WndWidth    : integer;
  end;

const
  C_StateTxt    : array[T_State] of String[15] = (
    '','','Searching...','Reading...','Saving...',
    'Printing...','Print Setup...');

  C_DflOptions  : T_Options =
  ( FileName    : '' ;
    OptDuration : true;
    OptToolbar  : true;
    OptHints    : true;
    WndLeft     : 20;
    WndTop      : 10;
    WndHeight   : 400;
    WndWidth    : 800
  );

var
  GV_ProgIdn   : String[8];
  GV_SettingFnm: TFileName;
  GV_CurDate   : T_DteStp;

procedure GetCurDate;
function  ValidDate(const V_Date: T_DteStp): boolean;
function  ValidTime(const V_Time: T_TimStp): boolean;
function  ToMinutes(const V_Time: T_TimStp; var V_Minutes: integer): boolean;
function  AddMinutes(const V_TimeFrom: T_TimStp; V_Minutes: integer;
                      var  V_TimeTo : T_TimStp): boolean;
function  Duration(const V_TimeFrom, V_TimeTo: T_TimStp): T_DurStp;
function  DisplayStp(const V_Str: StpTyp; V_OptDuration: boolean): StpTyp;
function  FullChnStp(const V_Chn: StpTyp; V_ChnList: TStringList): StpTyp;
function  SetListItem(const V_Date, V_TimeFrom, V_TimeTo,
                      V_Chn, V_Rec, V_Title: StpTyp): StpTyp;
function  GetListDate(const V_Item: StpTyp): T_DteStp;
function  GetListTimeFrom(const V_Item: StpTyp): T_TimStp;
function  GetListTimeTo(const V_Item: StpTyp): T_TimStp;
function  GetListChn(const V_Item: StpTyp): T_ChnStp;
function  GetListRec(const V_Item: StpTyp): T_RecStp;
function  GetListTitle(const V_Item: StpTyp): T_TtlStp;
function  SetChnListItem(const V_Ident, V_Descr: StpTyp): StpTyp;
function  GetChnListIdent(const V_Item: StpTyp): StpTyp;
function  GetChnListDescr(const V_Item: StpTyp): StpTyp;
function  GetRecCode(V_BtnYes,V_BtnNo,V_BtnNeutral: boolean): T_RecStp;
procedure ReadOptions(var V_Options: T_Options);
procedure WriteOptions(var V_Options: T_Options);


implementation
Uses
  RtvdMain,
  DtmLib, CvtLib, SckLib, StfLib, StpLib, NumLib,
  Dialogs, WinProcs;

procedure GetCurDate;
var DateRec: DtmDateRec; yyyy: String[4]; mm,dd: String[2];
begin with DateRec, Ymd do begin
  Fn:= DtmFnCur;
  if DtmConvert(DateRec) then begin
    ItoABL(year ,yyyy,10,4);
    ItoABL(month,mm  ,10,2);
    ItoABL(day  ,dd  ,10,2);
    GV_CurDate:= yyyy+'-'+mm+'-'+dd;
  end
  else GV_CurDate:= '';
end; end;

function  ValidDate(const V_Date: T_DteStp): boolean;
{ Validate 'yyyy-mm-dd' date string }
var DateRec: DtmDateRec;
begin
  with DateRec, Ymd do begin
    Fn   := DtmFnYmd;
    year := AtoIB(StfSub(V_Date,1,4),10);
    month:= AtoIB(StfSub(V_Date,6,2),10);
    day  := AtoIB(StfSub(V_Date,9,2),10);
  end;
  ValidDate:= DtmConvert(DateRec);
end;

function  ValidTime(const V_Time: T_TimStp): boolean;
{ Validate 'hh:mm' time string }
var minutes: integer;
begin
  ValidTime:= ToMinutes(V_Time, minutes);
end;

function  ToMinutes(const V_Time: T_TimStp; var V_Minutes: integer): boolean;
{ Validate 'hh:mm' time string; if valid: convert to minutes. }
var sh,sm: String[2]; hh,mm: integer;
begin
  sh:= StfSub(V_Time,1,2); hh:= AtoIB(sh,10);
  sm:= StfSub(V_Time,4,2); mm:= AtoIB(sm,10);
  if not ((sh='  ') or (sm='  '))
     and (    ((hh<24) and (mm<60))
           or ((hh=24) and (mm=00))
         )
  then begin
    V_Minutes:= 60*hh + mm;
    ToMinutes:= true;
  end
  else ToMinutes:= false;
end;

function AddMinutes(const V_TimeFrom: T_TimStp; V_Minutes: integer;
                     var  V_TimeTo  : T_TimStp): boolean;
{ Validate 'hh:mm' time string V_TimeFrom.
  If valid: add T_Minutes minutes, store result in V_TimeTo. }
var sh,sm: String[2]; hh,mm,minutes: integer;
begin
  if ToMinutes(V_TimeFrom,minutes) then begin
    minutes:= minutes + V_Minutes;
    hh:= (minutes div 60) mod 24;
    mm:= minutes mod 60;
    ItoABL(hh,sh,10,2);
    ItoABL(mm,sm,10,2);
    V_TimeTo:= sh+':'+sm;
    AddMinutes:= true;
  end
  else begin
    V_TimeTo:= ''; AddMinutes:= false;
  end;
end;

function Duration(const V_TimeFrom, V_TimeTo: T_TimStp): T_DurStp;
const MinutesPerDay = 24*60;
var   Minutes, Minutes1, Minutes2: integer; s: T_DurStp;
begin
  Minutes:= 0;
  if ToMinutes(V_TimeFrom,Minutes1) and ToMinutes(V_TimeTo,Minutes2)
  then begin
    Minutes:= Minutes2 - Minutes1;
    if Minutes < 0 then Inc(Minutes,MinutesPerDay);
    if Minutes > 999 then Minutes:= 999;
  end;
  if Minutes > 0 then ItoABLS(Minutes,s,10,C_DurLen)
                 else StpMake(s,' ',C_DurLen);
  Duration:= s;
end;

function  DisplayStp(const V_Str: StpTyp; V_OptDuration: boolean): StpTyp;
begin
  if not V_OptDuration then DisplayStp:= V_Str
  else DisplayStp:=
      StfSub(V_Str,1,C_ChnPos-1)
    + Duration(StfSub(V_Str,C_Tm1Pos,C_TimLen),
               StfSub(V_Str,C_Tm2Pos,C_TimLen))
    + ' '
    + StfSub(V_Str,C_ChnPos,MaxStp);
end;

function FullChnStp(const V_Chn: StpTyp; V_ChnList: TStringList): StpTyp;
var i: integer; FoundChn: boolean;
begin with V_ChnList do begin
  if Count = 0 then FullChnStp:= V_Chn
  else begin
    i:= -1; FoundChn:= false;
    while (not FoundChn) and (i < Count - 1) do begin
      Inc(i);
      FoundChn:= GetChnListIdent(V_ChnList[i]) = V_Chn;
    end;
   if FoundChn then FullChnStp:= V_ChnList[i] else FullChnStp:= V_Chn;
  end;
end; end;

function  SetListItem(const V_Date, V_TimeFrom, V_TimeTo,
                      V_Chn, V_Rec, V_Title: StpTyp): StpTyp;
begin
  SetListItem:=
    StfFill(StfTrunc(V_Date    ,C_DteLen),' ',C_DteLen)+' '+
    StfFill(StfTrunc(V_TimeFrom,C_TimLen),' ',C_TimLen)+'-'+
    StfFill(StfTrunc(V_TimeTo  ,C_TimLen),' ',C_TimLen)+' '+
    StfFill(StfTrunc(V_Chn     ,C_ChnLen),' ',C_ChnLen)+' '+
    StfFill(StfTrunc(V_Rec     ,C_RecLen),' ',C_RecLen)+' '+
    StfRTS          (V_Title);
end;

function  GetListDate(const V_Item: StpTyp): T_DteStp;
begin GetListDate:= StfSub(V_Item,C_DtePos,C_DteLen); end;

function  GetListTimeFrom(const V_Item: StpTyp): T_TimStp;
begin GetListTimeFrom:= StfSub(V_Item,C_Tm1Pos,C_TimLen); end;

function  GetListTimeTo(const V_Item: StpTyp): T_TimStp;
begin GetListTimeTo:= StfSub(V_Item,C_Tm2Pos,C_TimLen); end;

function  GetListChn(const V_Item: StpTyp): T_ChnStp;
begin GetListChn:= StfSub(V_Item,C_ChnPos,C_ChnLen); end;

function  GetListRec(const V_Item: StpTyp): T_RecStp;
begin GetListRec:= StfSub(V_Item,C_RecPos,C_RecLen); end;

function  GetListTitle(const V_Item: StpTyp): T_TtlStp;
begin GetListTitle:= StfSub(V_Item,C_TtlPos,C_TtlLen); end;

function SetChnListItem(const V_Ident, V_Descr: StpTyp): StpTyp;
begin
  SetChnListItem:=
    StfFill(StfTrunc(V_Ident,C_ChnLen),' ',C_ChnLen)+' '+V_Descr;
end;

function GetChnListIdent(const V_Item: StpTyp): StpTyp;
begin
  GetChnListIdent:= StfSub(V_Item,1,C_ChnLen);
end;

function GetChnListDescr(const V_Item: StpTyp): StpTyp;
begin GetChnListDescr:= StfSub(V_Item,C_ChnLen+2,C_TtlLen); end;

function  GetRecCode(V_BtnYes,V_BtnNo,V_BtnNeutral: boolean): T_RecStp;
begin
  if      V_BtnYes then GetRecCode:= '+'
  else if V_BtnNo  then GetRecCode:= '-'
  else                  GetRecCode:= '?';
end;

procedure ReadOptions(var V_Options: T_Options);
var ProgIdnZ, SettingFnmZ: array[0..255] of char;
    EmptyZ: array[0..1] of char; Value: StpTyp;
    bold, italic: boolean;
function ReadParm(const V_Key: StpTyp): boolean;
  var KeyZ, ValueZ: array[0..255] of char; ok: boolean;
  begin
    StrPCopy(KeyZ,V_Key);
    ok:= GetPrivateProfileString(
      ProgIdnZ, KeyZ, EmptyZ, ValueZ, 255, SettingFnmZ) > 0;
    if ok then Value:= StrPas(ValueZ) else Value:= '';
    ReadParm:= ok;
  end;
begin with V_Options do begin {ReadOptions}
  StrCopy(EmptyZ,'');
  StrPCopy(ProgIdnZ,GV_ProgIdn);
  StrPCopy(SettingFnmZ,GV_SettingFnm);
  if ReadParm('DataFile'    ) then FileName    := Value;
  if ReadParm('ShowDuration') then OptDuration := Value = '1';
  if ReadParm('ShowToolBar' ) then OptToolBar  := Value = '1';
  if ReadParm('ShowHints'   ) then OptHints    := Value = '1';
  if ReadParm('WndLeft'     ) then WndLeft     := StrToInt(Value);
  if ReadParm('WndTop'      ) then WndTop      := StrToInt(Value);
  if ReadParm('WndHeight'   ) then WndHeight   := StrToInt(Value);
  if ReadParm('WndWidth'    ) then WndWidth    := StrToInt(Value);
  with MainWnd.ListBox do begin
    bold:= false; italic:= false;
    if ReadParm('FontName')   then Font.Name   := Value;
    if ReadParm('FontSize')   then Font.Size   := StrToInt(Value);
    if ReadParm('FontColor')  then Font.Color  := StrToInt(Value);
    if ReadParm('FontBold')   then bold        := Value = '1';
    if ReadParm('FontItalic') then italic      := Value = '1';
    Font.Style:= [];
    if bold   then Font.Style:= Font.Style + [fsBold];
    if italic then Font.Style:= Font.Style + [fsItalic];
  end;
end; end;

procedure WriteOptions(var V_Options: T_Options);
var ProgIdnZ, SettingFnmZ: array[0..255] of char;
function WriteParm(const V_Key, V_Value: StpTyp): boolean;
  var KeyZ, ValueZ: array[0..255] of char;
  begin
    StrPCopy(KeyZ,V_Key);
    StrPCopy(ValueZ,V_Value);
    WriteParm:= WritePrivateProfileString(
      ProgIdnZ, KeyZ, ValueZ, SettingFnmZ);
  end;
function BoolToStr(V_Value: boolean): StpTyp;
  begin if V_Value then BoolToStr:= '1' else BoolToStr:= '0' end;
begin with V_Options, MainWnd.ListBox do begin {WriteOptions}
  StrPCopy(ProgIdnZ,GV_ProgIdn);
  StrPCopy(SettingFnmZ,GV_SettingFnm);
  if not (
    WriteParm('DataFile'     ,FileName)               and
    WriteParm('ShowDuration' ,BoolToStr(OptDuration)) and
    WriteParm('ShowToolBar'  ,BoolToStr(OptToolBar )) and
    WriteParm('ShowHints'    ,BoolToStr(OptHints   )) and
    WriteParm('WndLeft'      ,IntToStr (WndLeft    )) and
    WriteParm('WndTop'       ,IntToStr (WndTop     )) and
    WriteParm('WndHeight'    ,IntToStr (WndHeight  )) and
    WriteParm('WndWidth'     ,IntToStr (WndWidth   )) and
    WriteParm('FontName'     ,Font.Name)             and
    WriteParm('FontSize'     ,IntToStr(Font.Size))   and
    WriteParm('FontColor'    ,IntToStr(Font.Color))  and
    WriteParm('FontBold'     ,BoolToStr(fsBold   in Font.Style)) and
    WriteParm('FontItalic'   ,BoolToStr(fsItalic in Font.Style))
  ) then
      MessageDlg('Error writing ' + GV_SettingFnm,mtError, [mbOK], 0);
end; end;

end.
